package org.firstinspires.ftc.teamcode;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.SequentialAction;
import com.acmerobotics.roadrunner.SleepAction;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.eventloop.opmode.Autonomous;
import com.qualcomm.robotcore.eventloop.opmode.OpMode;
import com.qualcomm.robotcore.util.ElapsedTime;

import org.firstinspires.ftc.teamcode.cometbots.CometBotTeleopCompetition;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.subsystem.AutoPark;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath1;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath2;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath3;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath4;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath5;
import org.firstinspires.ftc.teamcode.subsystem.HighBasketAutoPath6;
import org.firstinspires.ftc.teamcode.subsystem.LiftActionsSubsystem;
import org.firstinspires.ftc.teamcode.subsystem.SkyHookSubsystem;


@Autonomous(name = "Auto Test Competition V2", group = "Dev")
public class ComeBotDriveDevV2 extends OpMode {
    private Follower follower;
    private int state;

    private HighBasketAutoPath1 path1;
    private HighBasketAutoPath2 path2;

    private HighBasketAutoPath3 path3;
    private HighBasketAutoPath4 path4;
    private HighBasketAutoPath5 path5;
    private HighBasketAutoPath6 path6;

    private AutoPark pathPark;
    private SkyHookSubsystem hook;
    private CometBotTeleopCompetition comp;
    private static ElapsedTime runtime;
    private static boolean initalized = false;
    private static boolean followingPath = false;
    private LiftActionsSubsystem liftActionsSubsystem;

    @Override
    public void init() {
        follower = new Follower(hardwareMap);
        follower.setMaxPower(.75);
        path1 = new HighBasketAutoPath1();
        path2 = new HighBasketAutoPath2();
        path3 = new HighBasketAutoPath3();
        path4 = new HighBasketAutoPath4();
        path5 = new HighBasketAutoPath5();
        path6 = new HighBasketAutoPath6();

        pathPark = new AutoPark();

        comp = new CometBotTeleopCompetition(hardwareMap, telemetry, gamepad1, gamepad2);
        comp.initCloseClaw();
        hook = new SkyHookSubsystem(hardwareMap);
        state = 0;
    }


    public void loop() {
        telemetry.addData("state", state);
        telemetry.addData("followingPath", followingPath);
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
        }
        switch (state) {
            case 0:
                moveToPathOneAndHighBucket();
                break;
            case 1:
                doArmThing();
                break;
            case 2:
                moveToPathTwoAndPickSampleUp();
                break;
            case 3:
                doPickUpThing();
                break;
            case 4:
                moveToBasketPath3();
                break;
            case 5:
                theArmThing();
                break;
            case 6:
                moveToPickupAgainPath4();
                break;
            case 7:
                doPickUpThingAgain();
                break;
            case 8:
                moveToPickupAgainPath5();
                break;
            case 9:
                theArmThingAgain();
                break;
            case 10:
                //line six
                break;
            case 11:


        }
        telemetry.update();
        follower.update();
    }

    private void moveToPathOneAndHighBucket() {
        if (!followingPath) {
            runtime = new ElapsedTime();
            path1.moveToPath1(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 4) {
                state = 1;
                followingPath = false;
            }
        }
    }

    public class SetStateAction implements Action {

        private int value;

        public SetStateAction(int value) {
            this.value = value;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            state = value;
            return false;
        }
    }

    private Action setStateValue(int value) {
        return new SetStateAction(value);
    }

    private void doArmThing() {
        comp.highBucketDrop();
        state = 2;
    }

    private void theArmThing() {
        telemetry.addData("busy?", follower.isBusy());
        telemetry.addData("end?", follower.atParametricEnd());
        if (follower.atParametricEnd()){
            follower.breakFollowing();
            comp.highBucketDrop();
            state = 6;
        }
//        follower.breakFollowing();
    }

    private void theArmThingAgain() {
        follower.breakFollowing();
        comp.highBucketDrop();
        state = 10;
    }

    private void moveToPathTwoAndPickSampleUp() {
        if (!followingPath) {
            path2.moveToPath2(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 22.0) {
                state = 3;
                followingPath = false;
            }
        }
    }

    private void moveToPickupAgainPath4() {
        if (!followingPath) {
            path4.moveToPickupAgainPath4(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 27.0) {
                state = 7;
                followingPath = false;
            }
        }
    }

    private void moveToPickupAgainPath5() {
        if (!followingPath) {
            path5.moveToPickupAgainPath5(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 36.0) {
                state = 9;
                followingPath = false;
            }
        }
    }
    private void moveToParkPath6() {
        if (!followingPath) {
            path6.moveToParkPath6(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 48.0) {
                state = 9;
                followingPath = false;
            }
        }
    }

    private void moveToBasketPath3() {
        if (!followingPath) {
            path3.moveToBasketPath3(follower);
            followingPath = true;
        }
        if (runtime != null) {
            telemetry.addData("Runtime (seconds)", runtime.seconds());
            if (follower.atParametricEnd() || runtime.seconds() > 12.0) {
                state = 5;
                followingPath = false;
            }
        }
    }

    private void thePickUp() {
        Actions.runBlocking(new SequentialAction(
                new SleepAction(.25),
                comp.arm.toSubmarinePosition(),
                new SleepAction(.5),
                comp.wrist.toPickupPosition(),
                new SleepAction(.75),
                comp.claw.closeClaw(),
                new SleepAction(.75),
                comp.wrist.toFloorPosition(),
                new SleepAction(0.5),
                comp.arm.toParkPosition(),
                new SleepAction(.25)
        ));
    }

    private void doPickUpThing() {
        follower.breakFollowing();
        thePickUp();
        state = 4;
    }

    private void doPickUpThingAgain() {
        follower.breakFollowing();
        thePickUp();
        state = 8;
    }


}

